@@ -1,6 +1,6 @@
 ;;; matlab-ts-mode--ei.el --- MATLAB electric indent -*- lexical-binding: t -*-
 
-;; Version: 8.0.1
+;; Version: 8.0.4
 ;; URL: https://github.com/mathworks/Emacs-MATLAB-Mode
 ;; SPDX-License-Identifier: GPL-3.0-or-later
 ;;
@@ -204,6 +204,10 @@
 
     ))
 
+(defun matlab-ts-mode--assert-msg (msg)
+  "Call error with MSG for code that shouldn't be hit."
+  (error "Assert: %s" msg))
+
 (defun matlab-ts-mode--ei-fast-back-to-indentation ()
   "Fast back to indentation.
 The `back-to-indentation' function uses the syntax table causes
@@ -321,7 +325,7 @@ is used in `matlab-ts-mode--ei-spacing'"
                    (next-type (treesit-node-type next-node)))
               ;; At comma node of a row in an array (a matrix or cell)?
               (when (and (equal next-type ",")
-                         (string= (treesit-node-type (treesit-node-parent next-node)) "row"))
+                         (string= (treesit-node-type (treesit-node-parent candidate)) "row"))
                 (cl-return-from matlab-ts-mode--ei-move-to-and-get-node
                   (cons next-node next-type))))
             (setq candidate (treesit-node-parent candidate)))))
@@ -365,9 +369,10 @@ is used in `matlab-ts-mode--ei-spacing'"
          (orig-line-no-spaces-or-commas (replace-regexp-in-string "[ \t,]" "" orig-line)))
 
     (when (not (string= new-line-no-spaces-or-commas orig-line-no-spaces-or-commas))
-      (error "Assert: line-content-no-space-mismatch new: \"%s\" !EQ orig: \"%s\" at line %d in %s"
-             new-line-no-spaces-or-commas orig-line-no-spaces-or-commas
-             (line-number-at-pos (point)) (buffer-name)))))
+      (matlab-ts-mode--assert-msg
+       (format "line-content-no-space-mismatch new: \"%s\" !EQ orig: \"%s\" at line %d in %s"
+               new-line-no-spaces-or-commas orig-line-no-spaces-or-commas
+               (line-number-at-pos (point)) (buffer-name))))))
 
 (defvar-local matlab--eilb nil) ;; Buffer used to create new electric indent line, ei-line
 
@@ -461,7 +466,7 @@ N-SPACES-TO-APPEND is the number of spaces to append between nodes."
 If an error node spans multiple lines, we ignore it assuming
 there's an inner error node.  This always returns non-nil, thus
 enabling caching."
-  (let ((error-linenums '(-1 . nil))
+  (let (error-linenums
         (capture-errors (when (not matlab-ts-mode--ei-tmp-buf-indent)
                           (treesit-query-capture (treesit-buffer-root-node)
                                                  matlab-ts-mode--ei-error-query))))
@@ -473,8 +478,11 @@ enabling caching."
              (error-end-linenum (line-number-at-pos error-end-pt)))
         (cl-loop
          for linenum from error-start-linenum to error-end-linenum do
-         (push `(,linenum . t) error-linenums))))
-    error-linenums))
+         (when (not (alist-get linenum error-linenums))
+           (push `(,linenum . t) error-linenums)))))
+    (if error-linenums
+        (reverse error-linenums)
+      '(-1 . nil))))
 
 (defun matlab-ts-mode--ei-no-elements-to-indent ()
   "Return t if no elements in the current line to indent.
@@ -501,19 +509,38 @@ Assumes that current point is at `back-to-indentation'."
       ;; Make invisible comma visible by returning it.
       ",")
 
-     ;; Case: invisible at end of array row
+     ;; Case: invisible comma to insert at at end of array row
      ;;           foo = {'one', 'two' ...
      ;;                              ^   missing comma, return a comma to have it inserted
-     ((string= node-type "line_continuation")
-      (let ((next-node (treesit-node-next-sibling node))
-            next-type)
+     ((and (string= node-type "line_continuation")
+           (string= (treesit-node-type (treesit-node-parent node)) "row"))
+      (let ((next-node (treesit-node-next-sibling node))  ;; after the line continuation
+            (ans ""))
         (while (and next-node
-                    (string= (setq next-type (treesit-node-type next-node)) "line_continuation"))
+                    (string= (treesit-node-type next-node) "line_continuation"))
           (setq next-node (treesit-node-next-sibling next-node)))
-        (if (and (equal next-type ",")
-                 (= (treesit-node-start next-node) (treesit-node-end next-node)))
-            '(",") ;; list means prefix to NODE text
-          "")))
+
+        (when next-node
+          (let ((row-item next-node)
+                (next-parent (treesit-node-parent next-node)))
+            (while (and next-parent
+                        (not (string= (treesit-node-type next-parent) "row")))
+              (setq row-item next-parent
+                    next-parent (treesit-node-parent next-parent)))
+
+            (when next-parent ;; next parent is a row
+              ;; Emacs 30.1 build 1 returns the hidden comma when using
+              ;; treesit-node-next-sibling. Emacs 30.1 build 2 doesn't, but does for
+              ;; treesit-node-prev-sibling.
+              (let ((row-item-type (treesit-node-type row-item)))
+                (when (not (string= row-item-type ","))
+                  (setq row-item (treesit-node-prev-sibling row-item))
+                  (setq row-item-type (treesit-node-type row-item)))
+                (if (and (equal row-item-type ",")
+                         (= (treesit-node-start row-item) (treesit-node-end row-item)))
+                    (setq ans '(","))  ;; list means prefix to NODE text
+                  )))))
+        ans))
 
      ;; Case: Handle ignored characters, e.g. ";" in matrices where node="]", next-node="["
      ;;   [[1, 2]; [3, 4]]
@@ -655,8 +682,9 @@ final amount of leading whitespace because we do electric indent before
                         (cl-return)))))
 
          (when (not n-spaces-between)
-           (error "Assert: ei, unhandled node <\"%s\" %S> and next-node <\"%s\" %S>"
-                  node-type node next-node-type next-node))
+           (matlab-ts-mode--assert-msg
+            (format "ei, unhandled node <\"%s\" %S> and next-node <\"%s\" %S>"
+                    node-type node next-node-type next-node)))
 
          (let* ((node-end (treesit-node-end node))
                 (next-node-start (treesit-node-start next-node))
@@ -667,9 +695,9 @@ final amount of leading whitespace because we do electric indent before
 
            (matlab--eilb-add-node-text node extra-chars n-spaces-between))
 
-           (setq node next-node
-                 node-type next-node-type)
-           ))
+         (setq node next-node
+               node-type next-node-type)
+         ))
 
       (when node ;; last node in line
         (when (equal start-node node)
@@ -760,35 +788,49 @@ Returns alist where each element in the alist is (COLUMN-NUM . WIDTH)"
 
     column-widths))
 
-(defun matlab-ts-mode--ei-get-m-matrix-row-in-line ()
+(defun matlab-ts-mode--ei-get-m-matrix-row-in-line (t-matrix-node)
   "Given point within a matrix assignment statement, return row node or nil.
 Example:  v = [1 2; ...
                ...
                3 4];
-when on the 2nd continuation only line, nil is returned otherwise a row node."
+when on the 2nd continuation only line, nil is returned otherwise a row node.
+If tmp T-MATRIX-NODE is non-nil, we use that to locate the first row."
   (save-excursion
-    (matlab-ts-mode--ei-fast-back-to-indentation)
-    (when (looking-at "[^ \t\n\r]")
-      (let (row-node
-            found-ans)
-        (cl-loop
-         while (not found-ans) do
+    (if t-matrix-node
+        ;; [1, 2, 3;      // child 0 is the '[', child 1 is the first row
+        ;;  4, 5, 6]
+        ;; [ ...          // child 0 is the '[', child 1 is line_continuation
+        ;;  1, 2, 3;      
+        ;;  4, 5, 6]
+        (let ((child1 (treesit-node-child t-matrix-node 1)))
+          (when (and (string= (treesit-node-type child1) "row")
+                     (= (line-number-at-pos (treesit-node-start t-matrix-node))
+                        (line-number-at-pos (treesit-node-start child1))))
+            ;; row-now on same line as use-matrix-node
+            child1))
+      ;; else on 2nd or later line, find the row
+      (matlab-ts-mode--ei-fast-back-to-indentation)
+      (when (looking-at "[^ \t\n\r]")
+        (let (row-node
+              found-ans)
+          (cl-loop
+           while (not found-ans) do
 
-         (let* ((node-at-pt (treesit-node-at (point) 'matlab))
-                (node node-at-pt))
-           (while (and node
-                       (not (string-match-p (rx bos (or "row" "matrix" "assignment") eos)
-                                            (treesit-node-type node))))
-             (setq node (treesit-node-parent node)))
+           (let* ((node-at-pt (treesit-node-at (point) 'matlab))
+                  (node node-at-pt))
+             (while (and node
+                         (not (string-match-p (rx bos (or "row" "matrix" "assignment") eos)
+                                              (treesit-node-type node))))
+               (setq node (treesit-node-parent node)))
 
-           (if (and node
-                    (string= (treesit-node-type node) "row"))
-               (setq found-ans t
-                     row-node node)
-             (goto-char (min (treesit-node-end node-at-pt) (pos-eol)))
-             (when (not (re-search-forward "[^ \t]" (pos-eol) t))
-               (setq found-ans t)))))
-        row-node))))
+             (if (and node
+                      (string= (treesit-node-type node) "row"))
+                 (setq found-ans t
+                       row-node node)
+               (goto-char (min (treesit-node-end node-at-pt) (pos-eol)))
+               (when (not (re-search-forward "[^ \t]" (pos-eol) t))
+                 (setq found-ans t)))))
+          row-node)))))
 
 ;; Internal variable that shouldn't be altered. It's used to avoid infinite recursion.
 (defvar matlab-ts-mode--ei-align-enabled t)
@@ -861,36 +903,40 @@ Returns the line number after the ASSIGN-NODE in the tmp-buf."
 (cl-defun matlab-ts-mode--ei-align-line-in-m-matrix (assign-node ei-info)
   "Align current line with EI-INFO in a multi-line matrix of ASSIGN-NODE.
 See `matlab-ts-mode--ei-get-new-line' for EI-INFO contents."
-  (let* ((assign-start-linenum (line-number-at-pos (treesit-node-start assign-node)))
-         (tmp-buf-ei-linenum (1+ (- (line-number-at-pos) assign-start-linenum)))
-         (tmp-buf-row-linenum (if matlab-ts-mode--ei-align-matrix-alist 1 tmp-buf-ei-linenum))
-         (matrix-alist matlab-ts-mode--ei-align-matrix-alist)
-         end-linenum)
+  (let* ((matrix-alist matlab-ts-mode--ei-align-matrix-alist)
+         (assign-start-linenum (line-number-at-pos (treesit-node-start assign-node)))
+         (t-buf-ei-linenum (1+ (- (line-number-at-pos) assign-start-linenum)))
+         (t-buf-row-linenum (if matlab-ts-mode--ei-align-matrix-alist 1 t-buf-ei-linenum)))
     (with-temp-buffer
       (unwind-protect
           (progn
-            (setq end-linenum (matlab-ts-mode--ei-indent-matrix-in-tmp-buf assign-node))
-            (let* ((assign-node (save-excursion
-                                  (matlab-ts-mode--ei-fast-back-to-indentation)
-                                  (treesit-parent-until (treesit-node-at (point) 'matlab)
-                                                        "assignment")))
-                   (matrix-node (if assign-node
-                                    (treesit-node-child-by-field-name assign-node "right")
-                                  (treesit-node-parent (treesit-search-subtree ;; else a property
+            (let* (t-curr-linenum
+                   (t-end-linenum (matlab-ts-mode--ei-indent-matrix-in-tmp-buf assign-node))
+                   (t-assign-node (save-excursion
+                                    (matlab-ts-mode--ei-fast-back-to-indentation)
+                                    (treesit-parent-until (treesit-node-at (point) 'matlab)
+                                                          "assignment")))
+                   (t-matrix-node (if t-assign-node
+                                      (treesit-node-child-by-field-name t-assign-node "right")
+                                    (treesit-node-parent (treesit-search-subtree ;; else a property
                                                         (treesit-buffer-root-node)
                                                         (rx bos "[" eos) nil t))))
-                   (first-col-extra (matlab-ts-mode--ei-m-matrix-first-col-extra matrix-node))
-                   (col-widths (matlab-ts-mode--ei-m-matrix-col-widths matrix-node
+                   (t-matrix-node-linenum (line-number-at-pos (treesit-node-start t-matrix-node)))
+                   (first-col-extra (matlab-ts-mode--ei-m-matrix-first-col-extra t-matrix-node))
+                   (col-widths (matlab-ts-mode--ei-m-matrix-col-widths t-matrix-node
                                                                        first-col-extra)))
               ;; Move to the line of interest when we called from matlab-ts-mode--treesit-indent,
               ;; otherwise calculate all matrix rows for indent-region.
               (when (and (not matrix-alist)
-                         (> tmp-buf-ei-linenum 1))
-                (forward-line (1- tmp-buf-ei-linenum)))
+                         (> t-buf-ei-linenum 1))
+                (forward-line (1- t-buf-ei-linenum)))
 
-              (while (< (line-number-at-pos) end-linenum) ;; Adjust column widths
+              ;; Adjust column widths
+              (while (< (setq t-curr-linenum (line-number-at-pos)) t-end-linenum)
                 (matlab-ts-mode--ei-fast-back-to-indentation)
-                (let* ((row-node (matlab-ts-mode--ei-get-m-matrix-row-in-line))
+                (let* ((row-node (when (>= t-curr-linenum t-matrix-node-linenum)
+                                   (matlab-ts-mode--ei-get-m-matrix-row-in-line
+                                    (when (= t-curr-linenum t-matrix-node-linenum) t-matrix-node))))
                        (indent-offset (when (looking-at "[^ \t\n\r]+") (current-column)))
                        ei-line)
                   (matlab--eilb-setup)
@@ -900,7 +946,7 @@ See `matlab-ts-mode--ei-get-new-line' for EI-INFO contents."
                              (r-content (buffer-substring indent-start-pt (pos-eol)))
                              (pt-offset (nth 1 ei-info))
                              (matrix-offset (save-excursion
-                                              (goto-char (treesit-node-start matrix-node))
+                                              (goto-char (treesit-node-start t-matrix-node))
                                               (1+ (- (point) (pos-bol))))))
                         ;; Adjust matrix row in eilb using r-content w/o leading indent-level spaces
                         (with-current-buffer matlab--eilb
@@ -911,9 +957,11 @@ See `matlab-ts-mode--ei-get-new-line' for EI-INFO contents."
                           (setq indent-offset matrix-offset))
 
                         (dolist (row-el (reverse (treesit-node-children row-node)))
-                          (when (not (string= (treesit-node-type row-el) ",")) ;; at a column?
+                          (when (not (string-match-p ;; at a column?
+                                      (rx bos (or "," "line_continuation" "comment") eos)
+                                      (treesit-node-type row-el)))
                             (let ((width (or (alist-get col-num col-widths)
-                                             (error "Assert: no col width")))
+                                             (matlab-ts-mode--assert-msg "no col width")))
                                   (curr-width (- (treesit-node-end row-el)
                                                  (treesit-node-start row-el))))
                               (when (< curr-width width)
@@ -940,20 +988,20 @@ See `matlab-ts-mode--ei-get-new-line' for EI-INFO contents."
                           (goto-char (point-max)))
 
                         (setq ei-line (matlab--eilb-content))
-                        (when (= tmp-buf-row-linenum tmp-buf-ei-linenum)
+                        (when (= t-buf-row-linenum t-buf-ei-linenum)
                           (setq ei-info (list ei-line pt-offset (nth 2 ei-info) (nth 3 ei-info)))))
 
                     ;; Else a blank or continuation line w/o matrix content
                     (setq ei-line (buffer-substring (pos-bol) (pos-eol))))
 
                   (when matrix-alist
-                    (let* ((buf-linenum (1- (+ assign-start-linenum tmp-buf-row-linenum))))
+                    (let* ((buf-linenum (1- (+ assign-start-linenum t-buf-row-linenum))))
                       (push `(,buf-linenum . ,ei-line) matrix-alist)))
 
                   (if (not matrix-alist)
                       (goto-char (point-max))
                     (forward-line)
-                    (setq tmp-buf-row-linenum (1+ tmp-buf-row-linenum)))))))
+                    (setq t-buf-row-linenum (1+ t-buf-row-linenum)))))))
         ;; unwind forms
         (matlab--eilb-kill)))
 
@@ -1044,7 +1092,7 @@ nil."
             ;; Case: unexpected matrix child node
             ((not (string-match-p (rx bos (or "[" "]" "comment" "line_continuation" "\n") eos)
                                   child-type))
-             (error "Assert: unexpected matrix child %S" child))))))
+             (matlab-ts-mode--assert-msg (format "unexpected matrix child %S" child)))))))
 
       ;; Multi-line matrix (m-matrix) with each row is on its own line?
       (let ((ans (and is-m-matrix (> n-rows 1) (>= n-cols 1))))
@@ -1255,7 +1303,7 @@ Note, \\='m-struct returns (list assignment-node max-field-width arguments-node)
                           (list assign-node (car pair) (cdr pair))))))))))
 
              (t
-              (error "Assert: bad a-type %S" a-type)))))))))
+              (matlab-ts-mode--assert-msg (format "bad a-type %S" a-type))))))))))
 
 (defun matlab-ts-mode--ei-point-in-m-type (ei-info m-type)
   "Are we in an M-TYPE line?
@@ -1285,8 +1333,10 @@ See `matlab-ts-mode--ei-get-new-line' for EI-INFO."
 
 (defun matlab-ts-mode--ei-assign-offset (ei-line)
   "Get the assignment offset from the indent-level in EI-LINE."
-  (let* ((first-char-offset (or (string-match-p "[^ \t]" ei-line) (error "Assert: no first char")))
-         (offset (- (or (string-match-p "=" ei-line) (error "Assert: no ="))
+  (let* ((first-char-offset (or (string-match-p "[^ \t]" ei-line)
+                                (matlab-ts-mode--assert-msg "no first char")))
+         (offset (- (or (string-match-p "=" ei-line)
+                        (matlab-ts-mode--assert-msg "no ="))
                     first-char-offset)))
     offset))
 
@@ -1376,7 +1426,7 @@ same line and has items to align."
   ;; becomes "nameValueArgs.foo (1,1)" in EI-LINE.
   (let ((ei-line (nth 0 ei-info)))
     (when (not (string-match "[^ \t]+" ei-line))
-      (error "Assert: no property in ei-line %s" ei-line))
+      (matlab-ts-mode--assert-msg (format "no property in ei-line %s" ei-line)))
     (- (match-end 0) (match-beginning 0))))
 
 ;; This is used to cache aligned properties/arguments for indent-region, and contains
@@ -1634,14 +1684,14 @@ TAB>  x = 123 ./1 + 567
         (when (string= curr-line-node-types orig-modified)
           (cl-return-from matlab-ts-mode--ei-assert-nodes-types-match))))
 
-    (error "Assert: line-node-types mismatch
+    (matlab-ts-mode--assert-msg (format "line-node-types mismatch
   new:  \"%s\"
   orig: \"%s\"
   at line %d in %s"
            curr-line-node-types
            orig-line-node-types
            linenum
-           (buffer-name))))
+           (buffer-name)))))
 
 (defun matlab-ts-mode--ei-assert-line-nodes-match (start-linenum end-linenum)
   "Assert that original line node types match modified line node types.
@@ -1736,7 +1786,7 @@ END-LINE1 corresponds to the first %-indent-mode=full comment and so on."
                      (push `(,start-line . ,(line-number-at-pos)) result)
                      (setq start-line nil)))
                   (_
-                   (error "Assert: bad directive, %s" directive)))))
+                   (matlab-ts-mode--assert-msg (format "bad directive, %s" directive))))))
             (when start-line
               (push `(,start-line . ,(line-number-at-pos (point-max))) result))))
         ;; Always return non-nil for caching and (-1 . 0) will never be used.
@@ -1966,5 +2016,5 @@ This expansion of the region is done to simplify electric indent."
 ;;; matlab-ts-mode--ei.el ends here
 
 ;; LocalWords:  SPDX gmail treesit defcustom bos eos isstring defun eol eobp setq curr cdr xr progn
-;; LocalWords:  listp alist dolist setf tmp buf utils linenum nums bobp pcase untabify SPC eilb
-;; LocalWords:  linenums reindent bol fubar repeat:ans defmacro bn
+;; LocalWords:  listp alist dolist setf tmp buf utils linenum nums bobp pcase untabify SPC eilb prev
+;; LocalWords:  linenums reindent bol fubar repeat:ans defmacro bn impl
